<?php

use EventEspresso\core\domain\entities\admin\menu\AdminMenuSubItem;

/**
 * Infusionsoft_Admin_Page_Init class
 * This is the init for the Infusionsoft Addon Admin Pages. See EE_Admin_Page_Init for method inline docs.
 *
 * @package               Event Espresso (infusionsoft addon)
 * @subpackage            admin/Infusionsoft_Admin_Page_Init.core.php
 * @author                Darren Ethier
 * ------------------------------------------------------------------------
 */

if (version_compare('4.4.0.dev.000', espresso_version(), '>')) {
    // BEFORE 4.4.0 when a new menu map system was introdcued
    class Infusionsoft_Admin_Page_Init extends EE_Admin_Page_Init
    {
        /**
         * Class constructor.
         */
        public function __construct()
        {
            do_action('AHEE_log', __FILE__, __FUNCTION__, '');
            define('INFUSIONSOFT_PG_SLUG', 'espresso_infusionsoft');
            define('INFUSIONSOFT_LABEL', esc_html__('Infusionsoft (Keap)', 'event_espresso'));
            define('EE_INFUSIONSOFT_ADMIN_URL', admin_url('admin.php?page=' . INFUSIONSOFT_PG_SLUG));
            define('EE_INFUSIONSOFT_ADMIN_ASSETS_PATH', EE_INFUSIONSOFT_ADMIN . 'assets' . DS);
            define('EE_INFUSIONSOFT_ADMIN_ASSETS_URL', EE_INFUSIONSOFT_URL . 'admin' . DS . 'infusionsoft' . DS . 'assets' . DS);
            define('EE_INFUSIONSOFT_ADMIN_TEMPLATE_PATH', EE_INFUSIONSOFT_ADMIN . 'templates' . DS);
            define('EE_INFUSIONSOFT_ADMIN_TEMPLATE_URL', EE_INFUSIONSOFT_URL . 'admin' . DS . 'infusionsoft' . DS . 'templates' . DS);
            parent::__construct();
            $this->_folder_path = EE_INFUSIONSOFT_ADMIN;
        }


        protected function _set_init_properties()
        {
            $this->label = INFUSIONSOFT_LABEL;
            $this->menu_label = INFUSIONSOFT_LABEL;
            $this->menu_slug = INFUSIONSOFT_PG_SLUG;
            $this->capability = 'administrator';
        }


        /**
         * Sets vars in parent for creating admin menu entry
         *
         * @return array
         */
        public function get_menu_map()
        {
            return [
                'group'        => 'settings',
                'menu_order'   => 25,
                'show_on_menu' => true,
                'parent_slug'  => 'events',
            ];
        }
    }
} else {
    // AFTER 4.4.0 when the new menu map system was introduced
    class Infusionsoft_Admin_Page_Init extends EE_Admin_Page_Init
    {
        /**
         * Class constructor.
         */
        public function __construct()
        {
            do_action('AHEE_log', __FILE__, __FUNCTION__, '');
            define('INFUSIONSOFT_PG_SLUG', 'espresso_infusionsoft');
            define('INFUSIONSOFT_LABEL', esc_html__('Infusionsoft (Keap)', 'event_espresso'));
            define('EE_INFUSIONSOFT_ADMIN_URL', admin_url('admin.php?page=' . INFUSIONSOFT_PG_SLUG));
            define('EE_INFUSIONSOFT_ADMIN_ASSETS_PATH', EE_INFUSIONSOFT_ADMIN . 'assets' . DS);
            define('EE_INFUSIONSOFT_ADMIN_ASSETS_URL', EE_INFUSIONSOFT_URL . 'admin' . DS . 'infusionsoft' . DS . 'assets' . DS);
            define('EE_INFUSIONSOFT_ADMIN_TEMPLATE_PATH', EE_INFUSIONSOFT_ADMIN . 'templates' . DS);
            define('EE_INFUSIONSOFT_ADMIN_TEMPLATE_URL', EE_INFUSIONSOFT_URL . 'admin' . DS . 'infusionsoft' . DS . 'templates' . DS);
            parent::__construct();
            $this->_folder_path = EE_INFUSIONSOFT_ADMIN;
        }


        protected function _set_init_properties()
        {
            $this->label = INFUSIONSOFT_LABEL;
        }


        protected function _set_menu_map()
        {
            $this->_menu_map = new AdminMenuSubItem([
                'menu_group'      => 'addons',
                'menu_order'      => 10,
                'show_on_menu'    => true,
                'parent_slug'     => 'espresso_events',
                'menu_slug'       => INFUSIONSOFT_PG_SLUG,
                'menu_label'      => esc_html__('Infusionsoft', 'event_espresso'),
                'capability'      => 'administrator',
                'admin_init_page' => $this,
            ]);
        }
    }
}
